// Yaniv Ovadia
// 20 February 2009

/*
 
 KEYWORDS:
 
 Scanner class for Java Input
 Graph algorithms - path-following
 
 */

import java.util.Scanner;   // for std in
import java.util.ArrayList;
import java.util.List;


/*
 * Algorithm Description:
 * Cycle over every vertex in the graph.
 * Follow the unique path from this vertex until a vertex is visited for a 
 * second time.
 * If this last vertex was part of the path above, then a cycle has been
 * discovered and its order dictates the longest path starting at any of its
 * vertices.
 * Vertices outside of a cycle will be referred to as tails and as one travels
 * along a tail, the longest path length decreases.
 **/

public class treat {
    
    public static void main (String[] args){
        Scanner stdin = new Scanner(System.in);
        
        // n = order of the graph
        // The graph is represented as an array where graph[i] = j implies that
        // vertex i has an edge to j. Note that every vertex has exactly one 
        // outgoing edge in this graph.
        int n = Integer.parseInt(stdin.nextLine());
        int[] graph = new int[n];

        // loop over lines in the input
        for (int i = 0; i < n; i++){
            // we prefer to index from 0, so the given vertices are 
            // subtracted by one
            graph[i] = Integer.parseInt(stdin.nextLine())-1;
        }
        
        // This constant temporarily holds a vertex's value in longestPath
        // until we have finished exploring and can set its value
        // This is useful for determining when we repeat a vertex.
        int PENDING = -1;

        int[] longestPath = new int[n]; // this is the array we will return
        boolean[] visited = new boolean[n];    // stores whether or not a
                                                // vertex has been visited
                                                // by either current or longest
                                                // path

        // loop over all vertices a in the graph
        for (int a=0; a < n; a++){
            if ( visited[a] ){continue; }  // skip this vertex if we already
                                            // have an answer for it


            // we then start from vertex "a" and follow it as far as possible
            // keeping track of a  "currentPath" list
            List<Integer> currentPath = new ArrayList();
            currentPath.add(a);
            int lastVertex = a;     // last vertex in currentPath
            int currentPathLen = 1; // length of currentPath

            // while we haven't yet reached a vertex for which we already have
            // an answer regarding the length of the longets path starting there
            // continue to add vertices to the currentPath
            while ( ! visited[lastVertex] ) {
                // we use PENDING as a bread crumb to know later that
                // this vertex has been visited by currentPath as opposed to
                // some prior longestPath
                longestPath[lastVertex] = PENDING;
                visited[lastVertex] = true;

                currentPath.add(graph[lastVertex]);
                lastVertex = graph[lastVertex];
                currentPathLen += 1;
            }
            
            // The length of our longest path is one less than the number
            // of vertices it contains.
            int longPathLen = currentPathLen - 1;
            
            // If the path ended at a vertex not in currentPath, then
            // we add the length of the longest path starting at that vertex.
            if (longestPath[lastVertex] != PENDING){
                longPathLen += longestPath[lastVertex]; }
            
            // We consider a vertex of currentPath to be a tail vertex if
            // the last vertex of its longest path is not itself.
            boolean isTail = true;

            // Now we go back over the path we just created to fill in
            // longestPath lengths
            for (int j=0; j< currentPathLen - 1; j++){
            
                // Notice that for non-tail vertices (vertices that are part
                // of a cycle), their longest path lengths are the same.
                int k = currentPath.get(j);
                if ( isTail )   { longestPath[k] = longPathLen-j; }
                else            { longestPath[k] = longPathLen; }
                
                // For tail vertices, the longest path decrements with
                // distance from the cycle
                if ( currentPath.get(currentPathLen-1) == k ){
                    isTail = false;
                    longPathLen -= j;
                }
            }

        }// for a

        // print result
        for (int i=0; i<n; i++){
            System.out.println(longestPath[i]);
        }


    }

}
